/*
 * Constants.h
 *
 *  Created on: 17 NOV 2016
 *      Author: Roman Ryutin
 */

#ifndef CONSTANTS_HH
#define CONSTANTS_HH

#include <cmath>
#include <complex>
#include <cstdlib>
#include <iostream>
#include <fstream>
#include <map>
#include <string>
#include <utility>
#include <vector>
#include <cassert>

#include "I.h"
#include "PI.h"

namespace ExDiff{

// aux class for constants definition
class Constants{
  public:
    Constants():
	 m_p(0.9383),
	 m_n(0.9396),
	 m_pi0(0.13498),
	 m_pi(0.1396),
	 m_H(125.0),
	 m_Gra(0.0),
	 m_R(300.0),
	 m_Glu(2.0),
	 m_Z(91.2),
	 m_W(80.4),
	 alpha_EM(1/137.036),
	 Lam_QCD(0.2),
	 VEV(246.0),
	 M_Pl(1.2209e+19),
	 IDproton(2212),
	 ID1710(10331),
	 ID1950(9050225),
	 ID1500(9030221),
	 IDpi0(111),
	 IDpiplus(211),
	 IDpiminus(-211),
	 IDdef(90),
	 ID1270(225),
	 ID2220(9000229)
	{}
	
//	Constants(Constants& constants_){}
    virtual ~Constants(){}
	
	void Print();
	inline double _mass(int ID_){
	   double mass_ = 0.0;	
	   if(ID_ == 2212){mass_ = m_p;}
	   if(ID_ == 2112){mass_ = m_n;}
       if(ID_ == 10331){mass_ = 1.72;}
	   if(ID_ == 9050225){mass_ = 1.95;}
	   if(ID_ == 9030221){mass_ = 1.5;}
	   if(ID_== 111){mass_ = m_pi0;}
	   if(ID_== 211 || ID_== -211){mass_ = m_pi;}
	   if(ID_== 90){mass_ = 125.0;}
	   if(ID_== 225){mass_ = 1.2751;}
	   if(ID_== 9000229){mass_ = 2.22;}
	   
	   return mass_;
	}
	
	inline int _dspin(int ID_){
	   int dspin_ = 1;	
	   if(ID_ == 2212){dspin_ = 2;}
	   if(ID_ == 2112){dspin_ = 2;}
       if(ID_ == 10331){dspin_ = 1;}
	   if(ID_ == 9030221){dspin_ = 1;}
	   if(ID_ == 9050225){dspin_ = 5;}
	   if(ID_== 111){dspin_ = 1;}
	   if(ID_== 211 || ID_== -211){dspin_ = 1;}
	   if(ID_== 90){dspin_ = 1;}
	   if(ID_ == 225){dspin_ = 5;}
	   if(ID_ == 9000229){dspin_ = 5;}
	   return dspin_;
	}	
	
// masses  
    const double m_p, m_n, m_pi0, m_pi, m_H, m_Gra, m_R, m_Glu, m_Z, m_W;
// couplings  
    const double  alpha_EM;
// scales
    const double Lam_QCD, VEV, M_Pl;	
// 
    const int IDproton, ID1710, ID1950, ID1500, IDpi0, IDpiplus, IDpiminus, IDdef, ID1270, ID2220;

	
//  private:
 	
}; // end of class Constants definitions

}// end of namespace ExDiff

#endif /* CONSTANTS_HH */
