/*
 * Event.h
 *
 *  Created on: 17 NOV 2016
 *      Author: Roman Ryutin
 */

#ifndef EVENT_HH
#define EVENT_HH

#include <cmath>
#include <complex> 
#include <cstdlib>
#include <iostream>
#include <fstream>
#include <map>
#include <string>
#include <utility>
#include <vector>
#include <cassert>
//////////////////////////
#include <iomanip>
#include <limits>

#include "I.h"
#include "PI.h"
#include "Particle.h"
#include "Constants.h"

namespace ExDiff{

// aux class for kinematics definition
class Event{
  public:
    Event():
 particles(Default())
{
	assert(!particles.empty());	
}
//		
    Event(const std::vector<Particle>& particles_):
 particles(particles_)
{
	assert(!particles.empty());	
}
//	
//	Event(Event& event_){}
    virtual ~Event(){}
	
	inline void Reset(const std::vector<Particle>& particles_){
		particles = particles_;
	}
	
// get particles const
    inline const std::vector<Particle>& _particles() const{
      return particles;
    };	
// get particles
    inline std::vector<Particle>& _particles(){
      return particles; 
    };		
		
	void Print(int);
	void AddToFile(int, std::string);
	std::vector<int> TakeFromFile(int, int, std::string);
        void TakeEvent(std::vector<int>, int, int, std::string);
    inline int CheckSum(){
        int n = 0; 
        int i = 0;		
    	double eps = 1.0e-10;
		std::vector<double> sum({0.0,0.0,0.0,0.0});
		for(i = 0; i < 2; i++){
			sum = particles[0].Lplus(sum,particles[i]._p());
		}
		for(i = 2; i < particles.size(); i++){
			sum = particles[0].Lminus(sum,particles[i]._p());
		}		
       
		for(i = 0; i < 4; i++){
          if(sum[i] > eps || sum[i] < -eps){n = 1;}		
		}
		
		return n;
	}
		
  private:
// vector of particles
   std::vector<Particle> particles;
//   
   inline std::vector<Particle> Default(){
	 Constants c;  
	 std::vector<double> v = {c.m_p,0.0,0.0,0.0};
	 Particle pa(v,c.m_p,2212,2,0,0);
     	 
     std::vector<Particle> p = {pa,pa,pa,pa};
	 
     return p;	 
   }

}; // end of class Event definitions

}// end of namespace ExDiff

#endif /* EVENT_HH */
