/*
 * NDgrid.h
 *
 *  Created on: 5 Nov. 2016 г.
 *      Author: Roman Ryutin
 */

#ifndef NDGRID_HH
#define NDGRID_HH


#include <cmath>
#include <complex>
#include <cstdlib>
#include <iostream>
#include <fstream>
#include <map>
#include <string>
#include <utility>
#include <vector>
#include <cassert>


namespace ExDiff{

template<typename T>
// grid of variables for a NDarray objects
class NDgrid{
  public:

    explicit NDgrid(const std::vector<std::size_t>& _dimensions) : 
	   dimensions(_dimensions), 
	   values(computeTotalSize(_dimensions)),
       point(initvector(_dimensions)),	   
	   pointIndexes(_dimensions)
	{
       assert(!dimensions.empty());
       assert(!values.empty());
	   assert(!point.empty());
	   assert(!pointIndexes.empty()); 
//       std::cout << " GRID CONSTRUCTOR print point vector = " << std::endl;
//       PrintVector(point,point.size());	   	   
    };

//    NDgrid(){}	
//	NDgrid(NDgrid& ndgrid_){}	
    virtual ~NDgrid(){};
	
/////////////////////////////////////////////////
// get const node point for any separate variable, use only this method to change the grid table manually !!!
    inline const T& GetVar(const std::size_t& varindex, const std::size_t& index) const{
      return values[computeIndex(varindex,index)];
    };
// get node point for any separate variable
    inline T& GetVar(const std::size_t& varindex, const std::size_t& index){
      return values[computeIndex(varindex,index)];
    };	
/////////////////////////////////////////////////	
// get dimensions const
    inline const std::vector<size_t>& GetDimensions() const{
      return dimensions;
    };	
// get dimensions
    inline std::vector<size_t>& GetDimensions(){
      return dimensions;
    };	
/////////////////////////////////////////////////	
// get values const
    inline const std::vector<T>& GetValues() const{
      return values;
    };	
// get values
    inline std::vector<T>& GetValues(){
      return values;
    };	
/////////////////////////////////////////////////	
// get const separate variable from the node point  
    inline const T& GetPointVar(const std::size_t& varindex) const{
      return point[varindex];
    };
// get separate variable from the node point
    inline T& GetPointVar(const std::size_t& varindex){
      return point[varindex];
    };		
/////////////////////////////////////////////////	
// get const index of separate variable from the node point  
    inline const std::size_t& GetPointVarIndex(const std::size_t& varindex) const{
      return pointIndexes[varindex];
    };
// get index of separate variable from the node point
    inline std::size_t& GetPointVarIndex(const std::size_t& varindex){
      return pointIndexes[varindex];
    };			
/////////////////////////////////////////////////
// get vector node point const 
    inline const std::vector<T> GPoint() const{  
      return point;
    };
// get vector node point
    inline std::vector<T> GPoint(){  
      return point;
    };	
/////////////////////////////////////////////////
// get vector of indexes of the node point  const
    inline const std::vector<size_t> GPointIndexes() const{  		
      return pointIndexes;
    };			
// get vector of indexes of the node point
    inline std::vector<size_t> GPointIndexes(){  		
      return pointIndexes;
    };
/////////////////////////////////////////////////	
// get corresponding vector node point
/*
    inline const std::vector<T>& get(const std::vector<std::size_t>& _indexes) const{      
	  std::vector<T> point_ = point;
      if(point_.size() < dimensions.size()){
		std::cout << "fill the GRID with numbers!" <<std::endl;  
	  } 	  
      if(point_.size() == dimensions.size()){	
        assert(point_.size() == _indexes.size());	
        for(std::size_t i=0; i < point_.size(); i++){
//		   pointIndexes[i]=_indexes[i];
		   point_[i]=GetVar(i,_indexes[i]);
 	    }	  
      }
      return &point_;
    };
*/
// get corresponding vector node point
    inline std::vector<T>& get(const std::vector<std::size_t>& _indexes) {
      if(point.size() < dimensions.size()){
		std::cout << "fill the GRID with numbers!" <<std::endl;  
	  } 	  
      if(point.size() == dimensions.size()){	
        assert(point.size() == _indexes.size());	
        for(std::size_t i=0; i < point.size(); i++){
		   pointIndexes[i]=_indexes[i];
		   point[i]=GetVar(i,_indexes[i]);
 	    }	  
      }
      return point;
    };		
// compute Index of the concrete variable node point in the NDgrid vector
    std::size_t computeIndex(const std::size_t& varindex, const std::size_t& index) const{
        assert(varindex < dimensions.size());
        assert(index < dimensions[varindex]);
		
        std::size_t indexx = index;
        std::size_t add = 0;
		
		if (varindex > 0){
          for (std::size_t i = 0; i < varindex; i++) {
             add += dimensions[i];
          }
		  indexx +=add;
		}
		
        assert(indexx < values.size());
        return indexx;
    };
	
// print information about NDgrid
    inline void printC(){
	 std::size_t n = dimensions.size();	
	 std::cout<<"NDgrid OBJECT INFORMATION ========================= " <<std::endl;
     std::cout<<"grid dimensions SIZE = "<< dimensions.size() <<std::endl;		
	 std::cout<<"grid dimensions vector " <<std::endl;
	 PrintVector(dimensions,n);
     std::cout<<"grid vector TOTAL SIZE = "<< values.size() <<std::endl;		
	 std::cout<<"grid vector[0]...[end] = "<< values[0] << " , ... , " << values[values.size()-1] << " }" <<std::endl;	
     std::cout<<"grid point TOTAL SIZE = "<< point.size() <<std::endl;		
	 std::cout<<"grid point vector" <<std::endl;
     PrintVector(point,n);	 
     std::cout<<"grid pointIndexes TOTAL SIZE = "<< pointIndexes.size() <<std::endl;		
	 std::cout<<"grid pointIndexes vector" <<std::endl;	
     PrintVector(pointIndexes,n);	 
	 std::cout<<"================================================== " <<std::endl;
     return;
   };
// print vector grid point and real constant vector grid point
    inline void printGP(){	
	  	  
	  std::cout << " { ";
          for (std::size_t j = 0; j < dimensions.size(); j++) {
             std::cout << point[j];
			 if(j<dimensions.size()-1) {
				 std::cout <<  " , ";
			 }
          }	  
	  std::cout << " } " << std::endl;	  
//	  	  
	  std::cout << " { ";
          for (std::size_t i = 0; i < dimensions.size(); i++) {
             std::cout << GetVar(i, pointIndexes[i]);
			 if(i<dimensions.size()-1) {
				 std::cout <<  " , ";
			 }			 
          }	  
	  std::cout << " } " << std::endl;		
      return;
    };	    
// Load values of the NDgrid from file
    inline void LoadFromFile(std::string filename){
        std::ifstream f;
		
        f.open(filename, std::ifstream::in);
        if (!f.is_open()) {
          std::cout << "Can not open a *.grid file!\n";   
		}
        T r;
// alternative to push_back
//		int i=0; // uncomment when use the alternative
 
		values.clear(); // comment when use the alternative
		
        while (!f.eof())
         {
          f>>r;
// alternative to push_back		  
//		  values[i]=r; // uncomment when use the alternative
//		  i++; // uncomment when use the alternative
          values.push_back(r); // comment when use the alternative
         }
       f.close();
//     point initialization after download
//////////////////////////////////////////
       point.clear();
	   pointIndexes.clear();
       for(std::size_t i=0; i < dimensions.size(); i++){
		  pointIndexes.push_back(0);
		  point.push_back(GetVar(i,0));
	   }	   
/////////////////////////////////////////       	   
       return;
	   
   };
// print vector
   inline void PrintVector(std::vector<size_t> vec_, std::size_t size_){		  	  
	  std::cout << " { ";
          for (std::size_t j = 0; j < size_; j++) {
             std::cout << vec_[j];
			 if(j<size_-1) {
				 std::cout <<  " , ";
			 }
          }	  
	  std::cout << " } " << std::endl;	  		
      return;
   };	
   inline void PrintVector(std::vector<double> vec_, std::size_t size_){		  	  
	  std::cout << " { ";
          for (std::size_t j = 0; j < size_; j++) {
             std::cout << vec_[j];
			 if(j<size_-1) {
				 std::cout <<  " , ";
			 }
          }	  
	  std::cout << " } " << std::endl;	  		
      return;
   };  

   // compute total number of values in the NDgrid
    inline std::size_t computeTotalSize(const std::vector<std::size_t>& _dimensions) const{
    	std::size_t totalSize = 1;

//        for (auto i : _dimensions) {
//          totalSize += i;
//        }
		
        for (std::size_t i=0; i < _dimensions.size(); i++) {
          totalSize += _dimensions[i];
		}
		
		totalSize -=1;
        return totalSize;
    };
  
  private:

    std::vector<std::size_t> dimensions; // dimensions of the NDgrid, example {3,4,3,7} or {100,200,40,50,80}
    std::vector<T> values;  // numbers of the array in the single vector values
	std::vector<T> point; // concrete grid point
	std::vector<std::size_t> pointIndexes; // indexes for concrete grid point
 
// arbitrary vector of dimensions.size()
    inline std::vector<T> initvector(const std::vector<std::size_t>& dimensions_){
		std::vector<T> v;
		v.clear();
        for(std::size_t i=0; i < dimensions_.size(); i++){
		   v.push_back(1);
 	    }		
		return v;
	}; 


}; // end of class NDgrid

}// end of namespace ExDiff

#endif /* NDGRID_HH */
