/* iFunc.h
 *
 *  Created on: 18 окт. 2016 г.
 *      Author: Roman Ryutin
 */

#ifndef IFUNC_HH
#define IFUNC_HH

#include <cmath>
#include <complex>
#include <cstdlib>
#include <iostream>
#include <fstream>
#include <map>
#include <string>
#include <utility>
#include <vector>
#include <cassert>

#include "I.h"
#include "PI.h"
#include "NDarray.h"
#include "NDgrid.h"

namespace ExDiff{

// multidimensional interpolating class of a data table
class iFunc{
  public:
    explicit iFunc (const NDarray<double>& , const NDgrid<double>& );

//	iFunc(){}
//	iFunc(iFunc& ifunc_){}
    virtual ~iFunc(){}

// aux function, print information about iFunc object	
    void printC();	
// print any vector of type double or size_t	
	void PrintVector(std::vector<size_t>, std::size_t);
	void PrintVector(std::vector<double>, std::size_t);
// basic methods
// calculation of the interpolation function	
    double Calc(std::vector<double>);

// access to the max-min values of the function	and private variables maxfunpoint, minfunpoint
	inline const double& GetMaxFun() const{
		return maxfunvalue;
	};
	inline double& GetMaxFun(){
		return maxfunvalue;
	};		
//
	inline const std::vector<double>& GetMaxFunPoint() const{
		return maxfunpoint;
	};
	inline std::vector<double>& GetMaxFunPoint(){
		return maxfunpoint;
	};	
//
	inline const std::vector<size_t>& GetMaxFunIndexes() const{
		return maxfunindexes;
	};
	inline std::vector<size_t>& GetMaxFunIndexes(){
		return maxfunindexes;
	};		
//
	inline const double& GetMinFun() const{
		return minfunvalue;
	};
	inline double& GetMinFun(){
		return minfunvalue;
	};		
//
	inline const std::vector<double>& GetMinFunPoint() const{
		return minfunpoint;
	};
	inline std::vector<double>& GetMinFunPoint(){
		return minfunpoint;
	};		
//
	inline const std::vector<size_t>& GetMinFunIndexes() const{
		return minfunindexes;
	};	
	inline std::vector<size_t>& GetMinFunIndexes(){
		return minfunindexes;
	};		
// access to other private variables
	inline const std::vector<double>& GetPoint() const{
		return point;
	};	
	inline std::vector<double>& GetPoint(){
		return point;
	};
//	
	inline const std::vector<double>& GetMinVars() const{
		return minvars;
	};
	inline std::vector<double>& GetMinVars(){
		return minvars;
	};
//	
	inline const std::vector<double>& GetMaxVars() const{
		return maxvars;
	};
	inline std::vector<double>& GetMaxVars(){
		return maxvars;
	};
//	
	inline const std::vector<std::size_t>& GetDimensions() const{
		return dimensions;
	};
	inline std::vector<std::size_t>& GetDimensions(){
		return dimensions;
	};	
//	
	inline const std::vector<std::size_t>& GetCubeIndexes() const{
		return cubeindex;
	};	
	inline std::vector<std::size_t>& GetCubeIndexes(){
		return cubeindex;
	};	
//	
	inline const std::vector<double>& GetCubeMin() const{
		return cubemin;
	};
	inline std::vector<double>& GetCubeMin(){
		return cubemin;
	};
//	
	inline const std::vector<double>& GetCubeMax() const{
		return cubemax;
	};
	inline std::vector<double>& GetCubeMax(){
		return cubemax;
	};	
//	
	inline const NDarray<double>& GetTab() const{
		return iTab;
	};
	inline NDarray<double>& GetTab(){
		return iTab;
	};	
//	
	inline const NDgrid<double>& GetGrid() const{
		return iGrid;
	};
	inline NDgrid<double>& GetGrid(){
		return iGrid;
	};		
//
// decrease one dimension of NDarray, if dimensions.size()>1
    NDarray<double> ReduceNDarray(NDarray<double>&);
    NDgrid<double> 	ReduceNDgrid(NDgrid<double>&);

  private:
    NDarray<double> iTab;
	NDgrid<double> iGrid;
	std::vector<std::size_t> dimensions;	
	NDarray<size_t> vbin;
	std::size_t vbinsize;  
    std::vector<double> minvars;
    std::vector<double> maxvars;
	std::vector<double> point;
	std::vector<std::size_t> cubeindex;
	std::vector<double> cubemin;
	std::vector<double> cubemax;	
	std::vector<size_t> maxfunindexes;
	std::vector<size_t> minfunindexes;	
	double maxfunvalue;
	std::vector<double> maxfunpoint;    
	double minfunvalue;
	std::vector<double> minfunpoint;
//		
	inline std::vector<std::size_t> CalcDim(const NDarray<double>& _iTab){
		return _iTab.GetDimensions();
	};	
	NDarray<size_t> CalcBin(const NDarray<double>&);
	std::vector<double> CalcMinVars(const NDgrid<double>&);
	std::vector<double> CalcMaxVars(const NDgrid<double>&);
	std::size_t CalcMax(const NDarray<double>&);
	std::size_t CalcMin(const NDarray<double>&);	
	
	int CheckPoint(std::vector<double>&);	
	double FunDif(std::size_t&, std::size_t&, std::vector<double>&);
	void CalcCube(std::vector<double>&);
	
}; // end of class iFunc definitions

}// end of namespace ExDiff

#endif /* IFUNC_HH */